@@ -43,7 +43,7 @@ struct _GladeNameContext {
 GladeNameContext *
 glade_name_context_new (void)
 {
-	GladeNameContext *context = g_new0 (GladeNameContext, 1);
+	GladeNameContext *context = g_slice_new0 (GladeNameContext);
 
 	context->name_allocators = g_hash_table_new_full (g_str_hash,
 							  g_str_equal,
@@ -65,7 +65,7 @@ glade_name_context_destroy (GladeNameContext *context)
 
 	g_hash_table_destroy (context->name_allocators);
 	g_hash_table_destroy (context->names);
-	g_free (context);
+	g_slice_free (GladeNameContext, context);
 }
 
 gchar *
@@ -117,10 +117,12 @@ glade_name_context_dual_new_name (GladeNameContext *context,
 				  const gchar      *base_name)
 {
 	GladeIDAllocator *id_allocator;
+	GList            *free_ids = NULL, *l;
 	const gchar      *number;
 	gchar            *name = NULL, *freeme = NULL;
-	guint             i = 1;
-	
+	guint             i;
+	gboolean          found = FALSE;
+
 	g_return_val_if_fail (context != NULL, NULL);
 	g_return_val_if_fail (another_context != NULL, NULL);
 	g_return_val_if_fail (base_name && base_name[0], NULL);
@@ -130,11 +132,11 @@ glade_name_context_dual_new_name (GladeNameContext *context,
 		--number;
 
 	if (*number)
-        {
+	{
 		freeme = g_strndup (base_name, number - base_name);
 		base_name = freeme;
 	}
-	
+
 	id_allocator = g_hash_table_lookup (context->name_allocators, base_name);
 
 	if (id_allocator == NULL)
@@ -143,15 +145,28 @@ glade_name_context_dual_new_name (GladeNameContext *context,
 		g_hash_table_insert (context->name_allocators,
 				     g_strdup (base_name), id_allocator);
 	}
-	
-	do
-        {
+
+	while (!found)
+	{
 		g_free (name);
 		i = glade_id_allocator_allocate (id_allocator);
 		name = g_strdup_printf ("%s%u", base_name, i);
-	} 
-	while (glade_name_context_has_name (context, name) ||
-	       glade_name_context_has_name (another_context, name));
+		
+		if (!(glade_name_context_has_name (context, name) ||
+		      glade_name_context_has_name (another_context, name)))
+			found = TRUE;
+		else
+			free_ids = g_list_prepend (free_ids, GUINT_TO_POINTER (i));
+	}
+
+	/* Release all the ids that were not hits */
+	for (l = free_ids; l; l = l->next)
+	{
+		i = GPOINTER_TO_UINT (l->data);
+		
+		glade_id_allocator_release (id_allocator, i);
+	}
+	g_list_free (free_ids);
 
 	g_free (freeme);
 	return name;
@@ -221,17 +236,12 @@ glade_name_context_release_name (GladeNameContext *context,
 	}
 	while (TRUE);
 
-	/* if there is a number - then we have to unallocate it... */
-	if (ch == 0) return;
-
-
 	base_name = g_strdup (name);
 	*(base_name + (first_number - name)) = 0;
-	
+
 	if ((id_allocator =
 	     g_hash_table_lookup (context->name_allocators, base_name)) != NULL)
 	{
-
 		id = (int) strtol (first_number, &end_number, 10);
 		if (*end_number == 0)
 			glade_id_allocator_release (id_allocator, id);
