@@ -65,7 +65,7 @@ project_changed (GladeProject      *project,
 		 GladeLabelEditor *label_editor)
 {
 	if (label_editor->modifying ||
-	    !GTK_WIDGET_MAPPED (label_editor))
+	    !gtk_widget_get_mapped (GTK_WIDGET (label_editor)))
 		return;
 
 	/* Reload on all commands */
@@ -283,7 +283,7 @@ markup_toggled (GtkWidget        *widget,
 
 	label_editor->modifying = TRUE;
 
-	glade_command_push_group (_("Setting %s to use a pango markup string"), label_editor->loaded_widget->name);
+	glade_command_push_group (_("Setting %s to use a Pango markup string"), label_editor->loaded_widget->name);
 
 	property = glade_widget_get_property (label_editor->loaded_widget, "pattern");
 	glade_command_set_property (property, NULL);
@@ -354,7 +354,7 @@ width_toggled (GtkWidget        *widget,
 
 	label_editor->modifying = TRUE;
 
-	glade_command_push_group (_("Setting %s to set desired width in charachters"), 
+	glade_command_push_group (_("Setting %s to set desired width in characters"), 
 				  label_editor->loaded_widget->name);
 
 	property = glade_widget_get_property (label_editor->loaded_widget, "max-width-chars");
@@ -385,7 +385,7 @@ max_width_toggled (GtkWidget        *widget,
 
 	label_editor->modifying = TRUE;
 
-	glade_command_push_group (_("Setting %s to set maximum width in charachters"), 
+	glade_command_push_group (_("Setting %s to set maximum width in characters"), 
 				  label_editor->loaded_widget->name);
 
 	property = glade_widget_get_property (label_editor->loaded_widget, "width-chars");
@@ -489,7 +489,7 @@ wrap_mode_toggled (GtkWidget        *widget,
 
 	label_editor->modifying = TRUE;
 
-	glade_command_push_group (_("Setting %s to use specific pango word wrapping"), 
+	glade_command_push_group (_("Setting %s to use specific Pango word wrapping"),
 				  label_editor->loaded_widget->name);
 
 	property = glade_widget_get_property (label_editor->loaded_widget, "single-line-mode");
@@ -589,10 +589,9 @@ append_label_appearance (GladeLabelEditor   *label_editor,
 	table_attach (table, GTK_WIDGET (eprop), 1, 3, label_editor->appearance_group);
 	label_editor->properties = g_list_prepend (label_editor->properties, eprop);
 
-	/* Add invisible eprops at the end of this table... */
-	gtk_widget_set_no_show_all (GTK_WIDGET (markup_property), TRUE);
-	gtk_widget_hide (GTK_WIDGET (markup_property));
-	table_attach (table, GTK_WIDGET (markup_property), 0, 4, label_editor->appearance_group);
+	/* Tie the invisible editor property to the table's life-cycle */
+	g_object_set_data_full (G_OBJECT (table), "glade-markup-property-destroy-me", 
+				markup_property, (GDestroyNotify)gtk_widget_destroy);
 }
 
 
@@ -718,10 +717,9 @@ append_label_wrapping (GladeLabelEditor   *label_editor,
 	table_attach (table, GTK_WIDGET (eprop), 1, row++, label_editor->wrap_group);
 	label_editor->properties = g_list_prepend (label_editor->properties, eprop);
 
-	/* Add invisible eprops at the end of this table... */
-	gtk_widget_set_no_show_all (GTK_WIDGET (single_line_eprop), TRUE);
-	gtk_widget_hide (GTK_WIDGET (single_line_eprop));
-	table_attach (table, GTK_WIDGET (single_line_eprop), 0, row, label_editor->wrap_group);
+	/* Tie the invisible editor property to the table's life-cycle */
+	g_object_set_data_full (G_OBJECT (table), "glade-single-line-property-destroy-me", 
+				single_line_eprop, (GDestroyNotify)gtk_widget_destroy);
 }
 
 GtkWidget *
